rep.row <- function(x, n){
  matrix(rep(x, each = n), nrow = n)
}

stats_bi <- function(bhat, se, true, alpha = .95, drop = 100,
                     lbb = NULL, ubb = NULL, alternative = FALSE, t = 1){
  # bhat           : bi estimated, matrix     S * N
  # standard errors: se(bi) estimated, matrix S * N
  # true parameter : N * 1
  # significance level
  # degree of freedom
  
  # drop samples for too high or low estimates, nan or na
  keep_row <- !apply(bhat, 1, 
                     function(x) any(is.nan(x) | is.na(x) | abs(x) > drop))
  #print(paste(sum(!keep_row), "samples were dropped"))
  bhat <- bhat[keep_row, ]
  se   <- se[keep_row, ]
  btrue <- rep.row(true, n = nrow(bhat))
  
  # summary
  av.bi <- colMeans(bhat)
  sd.bi <- apply(bhat, 2, function(x) sd(x))
  av.se <- colMeans(se)  
  
  # other measures  
  bias_1  <- mean(colMeans(abs(bhat - btrue)))
  bias_2  <- mean(abs(av.bi - true))
  sd.S    <- apply(bhat, 1, function(x) sd(x))
  mean.S  <- apply(bhat, 1, function(x) mean(x))
  
  # coverage
  qtile <- alpha + (1 - alpha) / 2
  if (is.null(ubb) & is.null(lbb)) {
    if (alternative) {
      #ub <- apply(bhat, 1, function(x) quantile(x, probs = qtile))
      #lb <- apply(bhat, 1, function(x) quantile(x, probs = (1 - qtile)))
      #lb.bi <- lb * matrix(1, nrow = nrow(bhat), ncol = ncol(bhat))
      #ub.bi <- ub * matrix(1, nrow = nrow(bhat), ncol = ncol(bhat))
      sigma_hat <- apply(bhat, 1, function(x) sd(x)) * matrix(1, nrow = nrow(bhat), ncol = ncol(bhat))
      lb.bi <- bhat - qnorm(qtile) * (sigma_hat  +  se) / sqrt(t)  
      ub.bi <- bhat + qnorm(qtile) * (sigma_hat  +  se) / sqrt(t)
    } else {
      lb.bi <- bhat - qnorm(qtile) * se
      ub.bi <- bhat + qnorm(qtile) * se
    }
  } else {
    lb.bi <- lbb[keep_row, ]
    ub.bi <- ubb[keep_row, ]
  }
  true.bi <- btrue >= lb.bi & btrue <= ub.bi
  coverage.ind <- colMeans(true.bi)
  
  
  
  S <- sum(keep_row)
  mc_lb <- coverage.ind - qnorm(qtile) * sqrt((coverage.ind * (1 - coverage.ind)) / S)
  mc_ub <- coverage.ind + qnorm(qtile) * sqrt((coverage.ind * (1 - coverage.ind)) / S)
  
  out <- list(av.bi = av.bi,
              sd.bi = sd.bi,
              av.se = av.se,
              bias_1 = bias_1,
              bias_2 = bias_2,
              sd.S  = sd.S, 
              mean.S = mean.S,
              S = S,
              coverage.ind = coverage.ind,
              av.coverage = mean(coverage.ind),
              av.lb = colMeans(lb.bi),
              av.ub = colMeans(ub.bi),
              av.bi = colMeans(bhat),
              mc.eb = cbind(mc_lb, mc_ub),
              lb.bi = lb.bi,
              ub.bi = ub.bi,
              bi    = bhat)
  return(out)
}

stats_class <- function(bhat, hat_class, true_class, Q = 2, drop = 100){
  # bhat           : bi estimated, matrix     S * N
  # standard errors: se(bi) estimated, matrix S * N
  # true parameter : N * 1
  # significance level
  # degree of freedom
  
  # drop samples for too high or low estimates, nan or na
  keep_row <- !apply(bhat, 1, 
                     function(x) any(is.nan(x) | is.na(x) | abs(x) > drop))
  hat_class <- hat_class[keep_row, ]
  class_true <- rep.row(true_class, n = nrow(hat_class))
  # proportion for each class
  pc <- c()
  for (c in 1:Q){
    pc <- c(pc, mean(apply(hat_class, 1, function(x) length(which(x == c)) / length(x)))) # proportion of class in each
  }
  
  # True class
  flag <- mean(rowMeans(class_true == hat_class))
  out <- list(pc = pc,
              flag = flag)
  return(out)
}

